/*!\file main.c
 *
 * \brief fonction main de dspoold et handleer de signaux
 */

#include "utils.h"
#include <sys/inotify.h>

extern int alone_locked;

static char buf[1024*64];   // 64K is raisonnable
static char lockname[1024]; // nom du fichier verou de l'application  
static int ino,watch;
char *path,*extent,*cmd;


/*!\brief handler pour la gestion de signaux entrainant la fermeture du programme
 */
void quit_handler()
	{
	if(watch)
		{
		if(inotify_rm_watch(ino,watch))
			{
			perror("inotify_rm_watch failed");
			exit(1);
			}
		}

	if(ino)
		{
		if(close(ino)<0)
			{
			perror("close inotify file handle failed");
			exit(1);
			}
		}

	if(alone_locked)
		{
		unlink(lockname);
		}

	exit(0);
	}

/*!\brief fonction de base
 *
 */
int main(int argc,char **argv)
	{
	DIR *dir;
	struct dirent *ent;
	int loop,extlen;

	if(argc != 4)
		{
		fprintf(stderr,"Usage: dspoold /path .extent script\n\n");
		return 0;
		}

	/* Il est important de se mettre dans le répertoire a scanner, ainsi les fichiers temporaires seront
	 * au même endroit, le fichier de lock aussi.
	 */
	path = argv[1];
	if(chdir(path)==-1)
		{
		perror("Can't chdir");
		exit(1);
		}

	extent = argv[2];
	extlen = strlen(extent);
	cmd = argv[3];

	if(*extent != '.')
		{
		fprintf(stderr,"Usage: dspoold /path .extent script\n.extent MUST BEGIN BY A DOT\n\n");
		return 1;
		}
	if(strlen(extent)>6)
		{
		fprintf(stderr,"Usage: dspoold /path .extent script\n.extent HAS 6 CHARACTERS MAX\n\n");
		return 1;
		}

	sprintf(lockname,"dspools%s.lock",extent);
	signal( SIGINT,  quit_handler ); // fermeture propre en cas de Control+C
	signal( SIGQUIT, quit_handler ); // fermeture propre en cas de fin de processus
	signal( SIGTERM, quit_handler ); // fermeture propre en cas de kill
	atexit(quit_handler);
	alone(lockname);

	if( (ino = inotify_init()) < 0 )
		{
		perror("inotify system init failed");
		exit(1);
		}

	if( (watch = inotify_add_watch(ino,".", IN_CLOSE_WRITE | IN_ONLYDIR)) < 0 )
		{
		perror("inotify system add_watch failed");
		exit(1);
		}

	/****************************************************************************************/

	if((dir=opendir(path))==NULL)
		{
		perror("Can't opendir");
		exit(1);
		}

	while((ent=readdir(dir)))
		{
		int size = strlen(ent->d_name);

		if(!strcmp( ent->d_name + size - extlen, extent)) 
			{
			static char sys[1024];
			sprintf(sys,"%s '%s/%s'",cmd,path,ent->d_name);
			fprintf(stderr,"SYSTEM : %s\n",sys);
			system(sys);
			}
		}
	closedir(dir);

	/****************************************************************************************/

	for(loop=0;;loop++)
		{
		int i,len;

		fprintf(stderr,"Debug : in loop %i (ino=%d,watch=%d)\n",loop,ino,watch);
		
		/*
		 * Nous n'avons pas besoin de select(), read restera en mode bloquant jusqu'au prochain event
		 * C'est ce que nous voulons, select() ne ferait que rajouter du code !
		 *
		 *  Une version plus complete comprendrait la gestion de lancement multiple (sur des multiprocesseur)
		 * et donc un wait3() dans la boucle, un spool des fichiers à traités, des timeout sur les system(),
		 * des stats en IPC.
		 */
		len = read(ino,buf,sizeof(buf));
		for(i=0;i<len;)
			{
			struct inotify_event *e = (struct inotify_event *)(buf + i);
			int size = strlen(e->name);
	
			fprintf(stderr,"ino=%d mask=%d cookie=%d len=%d name=%s\n",e->wd,e->mask,e->cookie,e->len,e->name);
			if(!strcmp( e->name + size - extlen, extent) && file_exist(e->name)==1) 
				{
				static char sys[1024];
				sprintf(sys,"%s '%s/%s'",cmd,path,e->name);
				fprintf(stderr,"SYSTEM : %s\n",sys);
				system(sys);
				}

			i += sizeof(struct inotify_event) + e->len ;
			}
		}

	return 0;
	}

